@@ -1436,10 +1436,11 @@ void
 optimize_type_argx(NODE *node)
 {
 	NODE *argx = node->v.argx.node;
+	int nargs = node->v.argx.nargs;
 	optimize(argx);
 	if (argx->type == node_type_number) {
 		node->type = node_type_arg;
-		node->v.arg.number = node->v.argx.nargs + 2;
+		node->v.arg.number = argx->v.number + nargs;
 	}
 }
 
