@@ -511,12 +511,6 @@ env_function_cleanup_add(eval_environ_t env, void *data, void (*func)(void *))
 {
 	env_cleanup_list_append(env->function_cleanup_list, data, func);
 }
-
-void
-env_environ_cleanup_add(eval_environ_t env, void *data, void (*func)(void *))
-{
-	env_cleanup_list_append(env->environ_cleanup_list, data, func);
-}
 
 /* A call to expand_dataseg (see below) invalidates any C variables that
    pointed to the dataseg before the call. To avoid dereferencing invalid 
@@ -635,50 +629,58 @@ prog_trace(eval_environ_t env, const char *fmt, ...)
 	       buf);
 }	
 
+void instr_builtin(eval_environ_t env);
 void instr_funcall(eval_environ_t env);
 void instr_locus(eval_environ_t env);
 
+static void
+stack_trace_entry(eval_environ_t env, prog_counter_t pc)
+{
+	int i;
+	char *name;
+	struct mu_locus_point *ploc = NULL, loc;
+	
+	if (mf_cell_instr(prog[pc-2]) == instr_funcall ||
+	    mf_cell_instr(prog[pc-2]) == instr_builtin) {
+		name = (char*)(env->dataseg + mf_cell_c_value(prog[pc-1], size));
+		pc -= 2;
+	} else {
+		name = "(in catch)";
+		pc -= 3;
+	}
+	
+	for (i = 0; i < 10; i++) {
+		if (pc > i + 3 && prog[pc - i - 3].c_instr == instr_locus) {
+			loc.mu_file = (char*)(env->dataseg
+					      + mf_cell_c_value(prog[pc - i - 2], size));
+			loc.mu_line = mf_cell_c_value(prog[pc - i - 1], size);
+			ploc = &loc;
+			break;
+		}
+	}
+		
+	if (ploc)
+		mu_error("%04lu: %s:%u: %s",
+			 (unsigned long) pc,
+			 ploc->mu_file, ploc->mu_line, name);
+	else
+		mu_error("%04lu: %s",
+			 (unsigned long) pc, name);
+}
+
 void
 runtime_stack_trace(eval_environ_t env)
 {
 	size_t base;
-	
+
 	mu_error(_("stack trace:"));
+	stack_trace_entry(env, env->pc);
 	for (base = env->base; base < datasize + env->stack_size - 4;
 	     base = mf_c_val(env->dataseg[base + 1], size) + base + 1) {
-		int i;
 		prog_counter_t pc = mf_c_val(env->dataseg[base + 2], ulong) - 1;
-		char *name;
-		struct mu_locus_point *ploc = NULL, loc;
-
 		if (pc < 2)
 			break; /*FIXME*/
-		if (mf_cell_instr(prog[pc-2]) == instr_funcall) {
-			name = (char*)(env->dataseg + mf_cell_c_value(prog[pc-1], size));
-			pc -= 2;
-		} else {
-			name = "(in catch)";
-			pc -= 3;
-		}
-
-		for (i = 0; i < 10; i++) {
-			if (pc > i + 3
-			    && prog[pc - i - 3].c_instr == instr_locus) {
-				loc.mu_file = (char*)(env->dataseg
-						   + mf_cell_c_value(prog[pc - i - 2], size));
-				loc.mu_line = mf_cell_c_value(prog[pc - i - 1], size);
-				ploc = &loc;
-				break;
-			}
-		}
-		
-		if (ploc)
-			mu_error("%04lu: %s:%u: %s",
-				 (unsigned long) pc,
-				 ploc->mu_file, ploc->mu_line, name);
-		else
-			mu_error("%04lu: %s",
-				 (unsigned long) pc, name);
+		stack_trace_entry(env, pc);
 	}
 	mu_error(_("stack trace finishes"));
 }
@@ -2356,6 +2358,7 @@ env_init(eval_environ_t env)
 	env->status = SMFIS_CONTINUE;
 	env->tos = datasize + env->stack_size - 1;
 	env->base = 0;
+	env->pc = 0;
 	mf_c_val(env->reg, long) = 0; //FIXME
 
 	env->matches = NULL;
