@@ -363,12 +363,13 @@ cmd_get(FILE *fp, int argc, char **argv)
 	for (i = 1; i < argc; i++) {
 		char *attr = argv[i];
 		char *val;
-		if (!strcasecmp(attr, "ehlo"))
+		if (!mu_c_strcasecmp(attr, "ehlo"))
 			val = ehlo_domain;
-		else if (!strcasecmp(attr, "mailfrom"))
+		else if (!mu_c_strcasecmp(attr, "mailfrom"))
 			val = mailfrom_address;
-		/* FIXME: Timeouts */
-		else
+		else if (!mu_c_strcasecmp(attr, "transcript"))
+			val = smtp_transcript ? "ON" : "OFF";
+		else /* FIXME: Timeouts */
 			val = NULL;
 		if (val)
 			writeout(fp, "* %s=%s\r\n", attr, val);
@@ -377,6 +378,50 @@ cmd_get(FILE *fp, int argc, char **argv)
 	return 0;
 }
 
+int
+cmd_set(FILE *fp, int argc, char **argv)
+{
+	int i;
+
+	for (i = 1; i < argc; i++) {
+		char *val = strchr(argv[i], '=');
+		if (!val) {
+			writeout(fp, "NO bad assignment %s\n", argv[i]);
+			return 1;
+		}
+		if (!mu_c_strncasecmp(argv[i], "ehlo", 4) ||
+		    !mu_c_strncasecmp(argv[i], "mailfrom", 8)) {
+			/* OK */;
+		} else if (!mu_c_strncasecmp(argv[i], "transcript", 10)) {
+			val++;
+			if (mu_c_strcasecmp(val, "ON") &&
+			    mu_c_strcasecmp(val, "OFF")) {
+				writeout(fp, "NO bad assignment %s\n", argv[i]);
+				return 1;
+			}
+		} else {
+			writeout(fp, "NO bad assignment %s\n", argv[i]);
+			return 1;
+		}
+	}
+			
+	for (i = 1; i < argc; i++) {
+		char *attr = argv[i];
+		char *val = strchr(attr, '=') + 1;
+
+		if (!mu_c_strncasecmp(attr, "ehlo", 4)) {
+			free(ehlo_domain);
+			ehlo_domain = mu_strdup(val);
+		} else if (!mu_c_strncasecmp(attr, "mailfrom", 8)) {
+			free(mailfrom_address);
+			mailfrom_address = mu_strdup(val);
+		} else if (!mu_c_strncasecmp(attr, "transcript", 10))
+			smtp_transcript = mu_c_strcasecmp(val, "ON") == 0;
+	}
+	writeout(fp, "OK\r\n");
+	return 0;
+}
+
 int
 cmd_sid(FILE *fp, int argc, char **argv)
 {
@@ -472,7 +517,7 @@ cmd_quit(FILE *fp, int argc, char **argv)
 static struct callout_command callout_command_tab[] = {
 	{ "VRFY", 2, 0, cmd_vrfy },
 	{ "GET", 2, 0, cmd_get },
-	/* FIXME: SET */
+	{ "SET", 2, 0, cmd_set },
 	{ "SID", 2, 2, cmd_sid },
 	{ "TIMEOUT", SMTP_NUM_TIMEOUT+1, SMTP_NUM_TIMEOUT+1, cmd_timeout },
 	{ "RUN", 1, 1, cmd_run },
